/*****************************************************
Project : Sosulka 20
Version : 1.0
Date    : 29.10.2013
Author  : Sergey                          
Company : Intellekt Dialog                

Chip type           : ATtiny13
Clock frequency     : 9,600000 MHz
*****************************************************/

#include <tiny13.h>
#include <stdint.h>
#include <delay.h>

#define DEVICE_NUMBER	0//0-9

//--- LED scan table ---
flash uint8_t scan_ddr[] = {
(1<<2)|(1<<0),//1
(1<<2)|(1<<1),//3
(1<<1)|(1<<0),//5
(1<<3)|(1<<1),//7
(1<<4)|(1<<1),//9
(1<<4)|(1<<2),//11
(1<<3)|(1<<0),//13
(1<<3)|(1<<2),//15
(1<<4)|(1<<0),//17
(1<<4)|(1<<3) //19
};

flash uint8_t scan_port[] = {
(1<<2)|(0<<0),//1 
(0<<2)|(1<<0),//2
(1<<2)|(0<<1),//3
(0<<2)|(1<<1),//4
(1<<1)|(0<<0),//5
(0<<1)|(1<<0),//6
(1<<3)|(0<<1),//7
(0<<3)|(1<<1),//8
(0<<4)|(1<<1),//9
(1<<4)|(0<<1),//10
(0<<4)|(1<<2),//11
(1<<4)|(0<<2),//12
(1<<3)|(0<<0),//13
(0<<3)|(1<<0),//14
(1<<3)|(0<<2),//15
(0<<3)|(1<<2),//16
(0<<4)|(1<<0),//17
(1<<4)|(0<<0),//18
(0<<4)|(1<<3),//19
(1<<4)|(0<<3) //20
}; 

//
#define METEOR_LEN	10
flash uint8_t meteor_tab[]={
0, //0 
1, //1
1, //2
2, //3
3, //4
5, //5
8, //6
12,//7
15,//8
1  //9
};


flash uint8_t delay_table[10]={
50, 100, 70, 60, 210, 180, 90, 255, 60, 130
};

flash uint8_t star_table[]={
0, 15, 17, 5, 19, 13, 8, 19, 10, 18, 4, 11, 16, 3, 12, 7, 14, 2, 9, 6 
};

#define FALSE	0
#define TRUE	(!FALSE)

#define TOTAL_LEDS	20
#define BRIGHT_MAX	15
uint8_t led_bright[21]; 

#define BTN_TRIG_VAL	6//button trigger value
uint8_t btn_cnt=0;//button press counter
volatile uint8_t btn_press = FALSE;

eeprom uint8_t effect_number = 0;//0-2
eeprom uint8_t effect_speed = 0;//0-2 

flash uint8_t eff0_speed[]={ 20, 6, 2 };
flash uint8_t eff1_speed[]={ 40, 30, 20 };
flash uint8_t eff2_speed[]={ 150, 110, 60 };

//-----------------------------------------------------------
//PCINT  Button press IRQ
//-----------------------------------------------------------
interrupt [PCINT0] void pin_change_isr(void)
{ 	
	if( PINB & (1<<5) )//PB5 is HI (L->H)
		{
		TCCR0B = 0;//stop counter
		if( btn_cnt >= BTN_TRIG_VAL )
			{
			//set next effect
			btn_press = TRUE;
			};
		}
	else  //PB5 is LOW (H->L)
		{
		btn_cnt = 0;
	     //Restart TIM0
		TCNT0 = 0;
		TCCR0B = 0x05;//start counter
		}	                    
	
	TIFR0 = (1<<1);// TOV0 clear flag 
}

//-----------------------------------------------------------
//TIM0 Overflow IRQ (~36Hz)
//-----------------------------------------------------------
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{ 
	if( (btn_cnt < BTN_TRIG_VAL) && (!(PINB & (1<<5))) )
		{
		++btn_cnt;
		}
	else
		{ 
		TCCR0B = 0;//stop counter
		};
}

void led_regen9( uint16_t count );

//-----------------------------------------------------------
//Generate pseudo-random value for delay
//-----------------------------------------------------------
uint16_t rnd_delay_val( void )
{
	static uint8_t delay_cnt = DEVICE_NUMBER;
	                                                                    
	if( delay_cnt > (sizeof(delay_table)-1) )
 		{ 
   		delay_cnt = 0;
     	};
	return (((uint16_t)delay_table[ delay_cnt++ ]) << 4);//generate delay interval
}

//-----------------------------------------------------------
//Main program
//-----------------------------------------------------------
void main(void)
{
int8_t led_cnt = 0;
uint16_t i;
int8_t j; 
uint8_t speed;
uint8_t eff_number, eff_speed;

// Crystal Oscillator division factor: 1
#pragma optsize-
CLKPR=0x80;
CLKPR=0x00;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 9,375 kHz(~36Hz)
// Mode: Normal top=FFh
// OC0A output: Disconnected
// OC0B output: Disconnected
TCCR0A=0x00;
TCCR0B=0x00;//0x05;

// Timer/Counter 0 Interrupt(s) initialization
TIFR0 = (1<<1);// TOV0 clear flag
TIMSK0 = (1<<1);//TOIE0 set, enable TIM0 overflow IRQ

// External Interrupt(s) initialization
// INT0: Off
// Interrupt on any change on pins PCINT0-5: On
GIMSK=(1<<5);//Enable PCIE
MCUCR=0x00;
PCMSK=(1<<5);//Set PCINT5
GIFR=(1<<5);//Clear PCIF  

// Analog Comparator initialization
// Analog Comparator: Off
ACSR=0x80;
ADCSRB=0x00;

#asm( "sei" );//enable IRQs

restart_effect:
//reinit leds
for( j=0; j<(TOTAL_LEDS+1); j++ )
	{ led_bright[j] = 0; };
//load data from eeprom
eff_number = effect_number;
eff_speed = effect_speed;
if( eff_number > 2 ) eff_number = 0;//set correct values
if( eff_speed > 2 ) eff_speed = 0;	

i=0;
led_cnt = 0;

//----- select effect ------
switch( eff_number )
	{ 
	case 0://-  
		speed = eff0_speed[ eff_speed ];
		while ( btn_press == FALSE)
      	{
      	if( led_bright[ led_cnt ] < BRIGHT_MAX )
      		{
      		++led_bright[ led_cnt ];
      		if( led_cnt > 0 )
      			{ 
      			--led_bright[ led_cnt-1 ];
      			};
      		i = speed;
      		}
      	else
      		{ 
      		++led_cnt;
      		if( led_cnt == 21 )
      			{ 
      			led_bright[ 20 ] = 0;
      			led_cnt = 0;
      			i = rnd_delay_val();
      			};
      		};	
      	
      	led_regen9( i );
      	};
		break;
	case 1://
		led_cnt=0-METEOR_LEN;//meteor
		speed = eff1_speed[ eff_speed ];
		while( btn_press == FALSE )
			{ 
 			for( j=0; j<METEOR_LEN; j++ )
  				{
    				if( ((j+led_cnt)>=0) && ((j+led_cnt)<=19) )
      				{
      				led_bright[ j+led_cnt ] = meteor_tab[ j ];
      				};
      			};
      		++led_cnt;

      		i = speed;
      		if( led_cnt >= (19+METEOR_LEN) )
      			{
      			led_cnt = 0-METEOR_LEN;
      			i = rnd_delay_val();
      			};
      	
      		led_regen9( i );	
			};
		break;
	//case 2://
	default: 
		led_cnt = DEVICE_NUMBER;
		speed = eff2_speed[ eff_speed ];
		while( btn_press == FALSE )
			{
			for(j=0; j<TOTAL_LEDS; j++)
				{
				if( j == star_table[ led_cnt ] )//  
					{
					if( led_bright[ j ] < BRIGHT_MAX )
						{
						++led_bright[ j ];
						}
					else //   -  
						{ 
						++led_cnt;
						if( led_cnt >= TOTAL_LEDS )
							led_cnt = 0;
						};
					}
				else //  
					{
					if( led_bright[ j ] > 0 )
						--led_bright[ j ];
					}
				};

			led_regen9( speed );
			};
		break;
	};
//---------------------	

btn_press = FALSE;
++eff_speed;
if( eff_speed > 2 )
	{
	eff_speed = 0;
	++eff_number;
	if( eff_number > 2 )
		{ 
		eff_number = 0;
		};
	};
//store effect data in EEPROM
effect_speed = eff_speed;
effect_number = eff_number;

goto restart_effect;
}

//-----------------------------------------------------------
//    ( 9 )
//: -  
//-----------------------------------------------------------
void led_regen9( uint16_t count )
{
	uint8_t b, i, j;
	uint8_t led_cnt;
#pragma optsize-
	do
		{
		led_cnt = 0;
		for(i=0; i<TOTAL_LEDS; i++)
      		{
      		PORTB = 0;
      		b = led_bright[ i ];
      		if( b>0 )
      			{
      			DDRB = scan_ddr[ i>>1 ];
      			++led_cnt;
      			for( j=BRIGHT_MAX; j>0; j-- )//software PWM
      				{ 
               		if( j == b ) { PORTB = scan_port[ i ]; };
      				delay_us( 8 );
      				};
      			delay_us( 8 );//bright=0
      			};    		
      		};
			
		PORTB = 0;
 		while( led_cnt < 9 )
 			{
 			delay_us( 15*8 );
 			++led_cnt;
 			};   
 		
 		--count;
 	}while(count > 0);
#pragma optsize+
}
